;------------------------------------------------------------------------------
; File Name   :	k6313G1
; Base Address:	0000h Range: B600h - BF00h Loaded length: 0900h
;
; Druckergrundkassette R0115, Programm K6313g1_com.tap
;
; reass: Volker Pohlers 27.08.2008, letzte nderung: 27.08.2008
;------------------------------------------------------------------------------

		cpu	Z80
		page	0

;------------------------------------------------------------------------------
; OS
;------------------------------------------------------------------------------

; RAM
IOBYT		equ	0004h		; I/O-Byte zur Zuweisung log.Gert phy.	Gert
LISW		equ	0015h		; Schalter fr Hardcopy. 0 kein Copy, 1 Copy
KEYBU		equ	0025h		; Tastaturpuffer
EOR		equ	0036h		; Zeiger auf letzte fr	Anwender freie Adresse
SYSR		equ	02C0h		; reserviert fr Systemerweiterung
ATTYL		equ	0EFE1h		; Adresse TTY-Treiber fr LIST
TXLST		equ	0EFEFh		; Adresse einer	Zeichenkette des aktuellen LIST-Gertes
BWS		equ	0EC00h		; Bildwiederholspeicher
;0Fh ASV: 	DB 0 		;Register fr A bei Eintritt in BOS
;2Fh PU: 	BER 1 		;Hilfszelle

; CALL5
PRNST		equ 	09h		; Ausgabe einer	Zeichenkette zu	CONST
RCONB		equ 	0Ah		; Eingabe einer	Zeichenkette von CONST
;0F006h 	; STATUS CONST
;0F009h		; EINGABE ZEICHEN VON CONST


;------------------------------------------------------------------------------
; Druckermodul
;------------------------------------------------------------------------------

SIODA		EQU	0B0H		; Datenadr SIO Kanal A
SIOKD		EQU	SIODA+2		; Kdoadr   - " -
CTC		EQU	0A8H		; Adr CTC Kanal 0

; Drucker mit Schnittstelle RS232C und DTR-Protokoll (Hardware-Protokoll)

; Baudrate:
;	BDRAT in Bd (50, 75, 100, 150, 200, 300, 600, 1200, 2400, 4800 oder 9600)
; 	Zeitkonstante fuer CTC: ZKCTC=9600/BDRAT
; Datenbit: DABI
;	7 Datenbit: DABI=20H
;	8 Datenbit: DABI=60H
; Paritaetsbit: PABI
;	ohne     Paritaet: PABI=2
;	gerade   Paritaet: PABI=3
;	ungerade Paritaet: PABI=1
; Stopbit: STPBI
;	1   Stopbit: STPBI=4
;	1.5 Stopbit: STPBI=8
;	2   Stopbit: STPBI=0CH

; K6311/12:
ZKCTC		EQU	1		; 9600 Bd
DABI		EQU	60H		; 8 Datenbit
PABI		EQU	2		; ohne Paritaet
STPBI		EQU	4		; 1 Stopbit


; K6311G1.COM
; 	Befehlssatz ISO-G: 
;	Einzelpunktmodus: "ESC K ...", Halbzeilenvorschub: "ESC [ 1 e"
;
; K6313G1.COM
; 	EPSON-Befehlssatz: 
;	Einzelpunktmodus: "ESC K ...", Halbzeilenvorschub: "ESC J ..."
;
; Es ist Endlospapier ohne Seiteneinteilung fest eingestellt
; 	linker Rand: 10 Zeichen
; 	Max. Zeilenlnge: 70 Zeichen
;
; Die Programme K6311G1.COM und K6313G1.COM belegen den Speicherplatz:
; 	B500H bis BEFFH und
; 	2C4H bis 2C5H.
; 	Der Speicherbereich von BF00H bis BFFFH bleibt fr ZM frei.
;
; Zustzliche Steuerfolgen (wie Breitdruck, Schrgdruck usw.) knnen nicht an den Drucker
; gesandt werden.

;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

		org 0B600h

start:
		jp	dini
name:		db "K6313G1 ",0
		db    0

;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

;Aufbau RAM-Speicher
;Lnge 172 Byte
;IX=(2C4)=(EOR)
;
;IX+0
;IX+1,2 akt. Pos. im Zeilenpuffer
;IX+3,4 Beginn Zeilenpuffer
;IX+5,6 Ende Zeilenpuffer
;IX+7,8 Merkzelle fr Zeilenpufferende (z.B. bei Wagenrcklauf)
;IX+9   Merkzelle fr Grafik (F0h Grafik ein, 0 Grafik aus)
;IX+10  Merkzelle fr CRLF, FFh -> letzten Zeichen war CR, sonst 0
;IX+11,12 s. prnzp2 + crlf
;IX+12..171 Zeilenpuffer (160 Zeichen)

; max. Zeilenlnge 140
; Lnge Zeilenpuffer 160
; Zeichen pro Zeile 70

;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

dini:		ld	hl, (TXLST)	; Zeichenkette f. LIST eintragen
		ld	de, name	; "K6313G1"
		and	a		; Cy = 0
		sbc	hl, de
		ld	(TXLST), de
		push	de
		jr	z, ini1		; Druckertreiber schon init.
;
		ld	hl, (EOR)
		push	hl
		ld	de, start
		and	a
		sbc	hl, de
		pop	hl
		jr	c, dini1
		ex	de, hl
		dec	hl
dini1:		push	hl
		ld	hl, 172
		ld	de, 100h
		and	a
		sbc	hl, de
		jr	nc, dini2
		ex	de, hl
dini2:		pop	de
		ex	de, hl
		and	a
		sbc	hl, de
		ld	(EOR), hl
		ld	(2C4h),	hl	; SYSR+4
		push	hl
		pop	ix
		ld	de, 12
		add	hl, de
		ld	(ix+4),	h	; Beginn Zeilenpuffer
		ld	(ix+3),	l
		ld	(ix+2),	h	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	l
		ld	de, 140		; max. Zeilenlnge
		add	hl, de
		ld	(ix+6),	h	; Ende Zeilenpuffer
		ld	(ix+5),	l
		ld	(ix+10), 0	; CR-Merker rcksetzen
;Programmierung CTC und SIO
		ld	hl, PROGT
		ld	c, CTC		; Prog. CTC
		ld	b, 3
		otir
		ld	c, SIOKD	; Prog. SIO
		ld	b, 9
		otir
;
		call	clrzp		; Zeilenpuffer mit Leerzeichen init.
;
ini1:		ld	hl, ttyl	; TTY-Treiber f. LIST
		ld	(ATTYL), hl
		ld	hl, 4		; LIST := ATTYP
		res	7, (hl)
		res	6, (hl)
		xor	a
		ld	(LISW), a	; kein Hardcopy
		call	status		; Statusabfrage
		ld	c, PRNST	; Ausgabe einer	Zeichenkette zu	CONST
		pop	de
		and	a
		call	nz, 5
		and	a		; Cy = 0, d.h. kein Fehler
		ret


;------------------------------------------------------------------------------
; Initialisierungstabelle CTC und SIO
;------------------------------------------------------------------------------

PROGT:
;CTC
		db 00000011b		; Reset
		db 00010111b		; Zeitgeber/DI/Vorteiler 16
		db ZKCTC		; Zeitkonstante
;SIO
		db 18h
		db 4			; Steuerregister
		db 40H+PABI+STPBI	; keine	Paritt,1 Stoppbit,Vorteiler 16,Monosynch-Mode
		db 1			; Interruptvektor
		db 0			; Kein Interrupt
		db 3			; Steuerung Empfnger
		db 11000000b		; 8bit,	Empf. aus
		db 5			; Steuerung Sender
		db DABI+8		; 8 bit, Sender	ein

;------------------------------------------------------------------------------
; TTY-Treiber f. LIST (logischer Treiber)
;
; Treibereingangsparameter:
; - A Kommando (siehe unten)
; - C Zeichen (bei Kommando AUSGABE)
;
; Treiberausgangsparameter:
; - CY Fehlerstatus
;
; Kommando
; 00 Abfrage Status
; 	Return:
; 	A Status
; 	0 kein Zeichen bei Eingabegert, nicht bereit bei Ausgabegert
; 	sonst Zeichen liegt an bei Eingabegert,
; 	(im installierten CRT-Treiber wird der Zeichencode bergeben)
; 02 Ausgabe Zeichen
; 	Eingang:
; 	C Zeichen
; FF Initialisieren/Rcksetzen des Gertes
;------------------------------------------------------------------------------

ttyl:		inc	a		; wenn A=FF -> Initialisieren
		jp	z, start
		dec	a
		jp	z, status	; wenn A=0 -> Statusabfrage
		dec	a
		dec	a
		jr	nz, ttylerr
		call	ttyl0		; wenn A=2 -> Ausgabe Zeichen
		ret	nc
;
ttylerr:	xor	a
		ld	(LISW), a	; kein Hardcopy
		scf
		ret

ttyl0:		push	ix
		ld	ix, (2C4h)	; SYSR+4
		ld	(ix+9),	0F0h	; Merkzelle fr Grafik
		ld	hl, ttylret
		push	hl		; RETURN-Adresse
		ld	a, c		; A=auszugebendes Zeichen
		ld	hl, ttyltab1	; Tabelle der Steuerzeichen
		ld	bc, 7
		cpir			; Steuerzeichen	suchen
		jr	z, ttyl1	; wenn A Steuerzeichen
		dec	bc		; bc :=	-1, wenn kein Steuerz., so Funktion char_xx
;
ttyl1:		ld	hl, 6
		and	a
		sbc	hl, bc
		add	hl, hl
		ld	bc, ttyltab2	; Tabelle der Steuerzeichen-Funktionen
		add	hl, bc
		ld	c, (hl)
		inc	hl
		ld	h, (hl)
		ld	l, c
		jp	(hl)		; Steuerzeichenfunktion	starten
;
ttylret:	pop	ix
		ret

; Tabelle der Steuerzeichen
ttyltab1:	db 0Dh
		db 0Ah
		db 0Ch
		db 9
		db 17h
		db 0Eh
		db 8
; Tabelle der Steuerzeichen-Funktionen
ttyltab2:	dw char_0d		; Wagenrcklauf CR
		dw char_0a		; Cursor down
		dw char_0c		; CRLF (CLS?)
		dw char_09		; Cursor right
		dw char_0c		; CTRL/W Seitenvorschub	am Drucker
		dw char_0e		; CTRL/N Bildschirmkopie
		dw char_08		; Cursor left
		dw char_xx		; alle anderen Zeichen


;------------------------------------------------------------------------------
; Steuerzeichenfunktionen
;------------------------------------------------------------------------------

; Wagenrcklauf CR
char_0d:	ld	(ix+10), 0FFh	; CR merken
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	(ix+8),	h	; merken
		ld	(ix+7),	l
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	(ix+2),	h	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	l
		and	a		; Cy := 0
		ret

; Cursor down LF
char_0a:	bit	0, (ix+10)	; war CR?
		jr	nz, char_0a_1	; wenn ja
;
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	(ix+8),	h	; merken
		ld	(ix+7),	l
char_0a_1:	ld	(ix+10), 0	; CR-Merker rcksetzen
		call	prepzp		; Zeilenpuffer ausgeben
		jp	clrzp

; CLS -> CRLF
char_0c:	call	char_0d		; CR
		jp	char_0a		; LF

; Cursor right
char_09:	ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	d, (ix+6)	; Ende Zeilenpuffer
		ld	e, (ix+5)
		and	a		; Cy := 0
		sbc	hl, de		; Zeilenende erreicht?
		ccf
		call	z, char_0c	; ja -> CRLF
		ld	(ix+10), 0	; CR-Merker rcksetzen
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		inc	hl
		ld	(ix+2),	h	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	l
		ret

; sonstige Zeichen
char_xx:	cp	20h
		jr	nc, char_xx_1	; Steuerzeichen < 20h bergehen
		and	a		; Cy := 0
		ret
;
char_xx_1:	push	af
		call	char_09		; Cursor right
		pop	de
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	(hl), d		; Zeichen im Zeilenpuffer eintragen
		ret


; CTRL/N Bildschirmkopie
char_0e:	ld	(ix+9),	0	; Merkzelle fr Grafik
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	de, 40
		add	hl, de
		ld	(ix+6),	h	; Ende Zeilenpuffer
		ld	(ix+5),	l	; auf 40 Zeichen/Zeile verkleinern
;
		ld	de, BWS		; Bildwiederholspeicher
char_0e_1:	ld	a, (de)
		push	de
		call	char_xx		; Zeichen ausgeben (logisch)
					; Zeilenwechsel erfolgt automatisch
		pop	de
		ret	c
		inc	de
		ld	hl, BWS+40*24-1	; BWS-Ende
		and	a
		sbc	hl, de
		jr	nc, char_0e_1
;
		call	char_0c		; CRLF
		ret	c
;
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	de, 140
		add	hl, de
		ld	(ix+6),	h	; Ende Zeilenpuffer
		ld	(ix+5),	l	; auf 140 Zeichen/Zeile restaurieren
		ret

; Cursor left
char_08:	ld	(ix+10), 0	; CR-Merker rcksetzen
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	d, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	e, (ix+1)
		and	a
		sbc	hl, de		; Am Anfang der Zeile?
		ret	z		; dann gibt es nicht zu tun
		dec	de		; sonst 1 Zeichen nach links
		ld	(ix+2),	d	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	e
		and	a
		ret


;------------------------------------------------------------------------------
; Zeilenpuffer-Routinen
;------------------------------------------------------------------------------

; Zeilenpuffer mit Leerzeichen init.
clrzp:		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	b, 160		; Lnge Zeilenpuffer
clrzp1:		inc	hl
		ld	(hl), ' '
		djnz	clrzp1
		ret
; Zeilenpuffer fr Ausdrucken vorbereiten
prepzp:		ld	h, (ix+8)	; Merkzelle fr Zeilenpufferende
		ld	l, (ix+7)
		push	hl
		ld	b, (ix+4)	; Beginn Zeilenpuffer
		ld	c, (ix+3)
		and	a
		sbc	hl, bc
		ld	b, h		; BC := HL := Anzahl auszugebender Zeichen
		ld	c, l
		ex	(sp), hl	; HL := Zeilenpufferende
		push	hl
		jr	z, prepzp1	; wenn keine Zeichen auszugeben
; sonst Zeilenpuffer 1 Zeichen nach rechts verschieben
		ld	d, h
		ld	e, l
		inc	de
		lddr
prepzp1:	inc	hl
		ld	(hl), 0Dh	; als erstes Zeichen CR eintragen
		pop	hl		; HL := Zeilenpufferende
prepzp2:	inc	hl
		ex	(sp), hl
		ld	bc, 70		; Zeichen pro Zeile
		and	a
		sbc	hl, bc
		ld	b, h
		ld	c, l
		ex	(sp), hl
		jr	z, prepzp3
		jr	c, prepzp3
; wenn Zeilenlnge erreicht, so LFCR einfgen
		push	hl
		ld	d, h
		ld	e, l
		inc	de
		inc	de
		lddr
		inc	hl
		ld	(hl), 0Ah
		inc	hl
		ld	(hl), 0Dh
		pop	hl
		inc	hl
		jr	prepzp2
;
prepzp3:	pop	de
		inc	hl
		ld	(hl), 0Ah	; LF einfgen
		ld	(ix+8),	h	; Merkzelle fr Zeilenpufferende
		ld	(ix+7),	l

; Ausdrucken einer Zeile
		ld	b, (ix+4)	; Beginn Zeilenpuffer
		ld	c, (ix+3)
;
prnzp1:		ld	h, (ix+8)	; Merkzelle fr Zeilenpufferende
		ld	l, (ix+7)
		and	a
		sbc	hl, bc
		ret	z		; wenn kein Zeichen auszugeben ist
;
		push	bc
		pop	de
		ld	hl, 0FFFFh	; -1
		inc	de
prnzp2:		inc	de
		inc	hl
		ld	a, (de)
		cp	0Ah
		jr	nz, prnzp2	; weiter bis LF
;
		add	hl, hl
		push	hl
		add	hl, hl
		pop	de
		add	hl, de
		ld	(ix+12), h
		ld	(ix+11), l
		bit	7, (ix+9)	; Merkzelle fr Grafik
		jr	z, prngr	; Grafikzeichenausgabe
;
prnzp3:		inc	bc
		ld	e, 0
prnzp4:		ld	a, (bc)
		call	crlf
		ret	c
		call	dout		; Ausgabe D
		ret	c
		inc	e
		ld	a, 6
		cp	e
		jr	nz, prnzp4
		ld	a, (bc)
		cp	0Ah
		jr	nz, prnzp3
		jr	prnzp1


prngr:		push	bc		; Grafikzeichenausgabe
		inc	(ix+9)		; Merkzelle fr Grafik
prngr1:		inc	bc
		ld	e, 0
prngr2:		ld	a, (bc)
		cp	0Dh
		jr	nz, prngr3
		call	crlf
		jr	c, prngr13
		jr	prngr9
;
prngr3:		cp	0Ah
		jr	z, prngr10
		call	chrpos		; Position in Grafikzeichentabelle berechnen
		ld	a, (ix+9)	; Merkzelle fr Grafik
		cp	2
		jr	c, prngr5
		jr	z, prngr6
		cp	4
		jr	c, prngr4
		jr	z, prngr7
		cp	6
		jr	z, prngr8
		srl	d
		srl	d
prngr4:		srl	d
		srl	d
prngr5:		ld	a, d
		and	0FCh
		ld	d, a
		jr	prngr9
;
prngr6:		sla	d
		sla	d
prngr7:		sla	d
		sla	d
prngr8:		sla	d
		sla	d
;
prngr9:		call	dout
		jr	c, prngr13
		inc	e
		ld	a, 6
		cp	e
		jr	nz, prngr2
		jr	prngr1
;
prngr10:	ld	a, (ix+9)	; Merkzelle fr Grafik
		bit	0, a
		jr	z, prngr11
		call	escj		; Zeilenvorschub 1/12 Zoll
		jr	c, prngr13
		pop	bc
		jr	prngr		; Grafikzeichenausgabe
;
prngr11:	cp	6
		jr	nz, prngr12
		ld	(ix+9),	0	; Merkzelle fr Grafik
		call	escj		; Zeilenvorschub 1/12 Zoll
		jr	c, prngr13
prngr12:	pop	hl
		jp	prnzp1
;
prngr13:	pop	bc
		ret

;
crlf:		cp	0Ah
		ld	d, a
		jr	z, crlf2
		cp	0Dh
		jr	nz, chrpos	; Position in Grafikzeichentabelle berechnen
		call	dout		; Ausgabe D
		ret	c
		ld	e, 0Ah
crlf1:		ld	d, ' '
		call	dout		; Ausgabe D
		ret	c
		dec	e
		jr	nz, crlf1
;ESC K n1 n2  Single density graphics (60 dpi)
 		ld	d, 1Bh		; ESC
		call	dout		; Ausgabe D
		ret	c
		ld	d, 'K'
		call	dout		; Ausgabe D
		ret	c
		ld	d, (ix+11)
		call	dout		; Ausgabe D
		ret	c
		ld	d, (ix+12)
crlf2:		ld	e, 5
		and	a
		ret

; Position in Grafikzeichentabelle berechnen
chrpos:		ld	l, a
		ld	h, 0
		push	bc
		ld	bc, 20h
		and	a
		sbc	hl, bc
		add	hl, hl
		push	hl
		add	hl, hl
		pop	bc
		add	hl, bc
		ld	bc, chrset	; Grafikzeichensatz 6x8	(ab 20h)
		add	hl, bc
		pop	bc
		ld	d, 0
		add	hl, de
		ld	d, (hl)
		and	a
		ret

; Ausgabe ESC J	18
; ESC J n  Immediate n/216 inch line feed (n=0..255)
escj:		ld	d, 1Bh		; ESC
		call	dout
		ret	c
		ld	d, 'J'
		call	dout
		ret	c
		ld	d, 18		; 1/12 inch
		jp	dout

;------------------------------------------------------------------------------
; phys. Treiber
;------------------------------------------------------------------------------

; Ausgabe D
dout:		push	bc
		push	de
		push	hl
		ld	hl, (2Fh)
		ld	a, (0Fh)
		ld	h, a
		push	hl
		call	0F006h		; STATUS CONST
		jr	c, dout1
		and	a
		jr	z, dout1
		cp	3
		scf
		jr	z, dout1
		xor	10h
		jr	nz, dout1
		xor	a		; A := 0
		ld	(LISW), a	; kein Hardcopy
		call	0F009h		; EINGABE ZEICHEN VON CONST
dout1:		pop	hl
		ld	a, h
		ld	(0Fh), a
		ld	a, l
		ld	(2Fh), a
		pop	hl
		pop	de
		pop	bc
		ret	c
		call	status
		and	a
		jr	z, dout
		ld	a, d
		out	(SIODA), a
		ret

; Statusabfrage
status:		ld	a, 10h
		out	(SIOKD), a
		in	a, (SIOKD)
		and	00100100b
		cp	00100100b
		ret	z
		xor	a
		ret

;------------------------------------------------------------------------------
; Zeichensatz 6x8 Pixel, 224 Zeichen ab 20h
;------------------------------------------------------------------------------

chrset:		equ	$
		include	zsatz_td40.inc

		end

