;------------------------------------------------------------------------------
; File Name   :	td40.rom (M506)
; Base Address:	0000h Range: B800h - C000h Loaded length: 0800h
;
; Druckermodul 690006.8 f. Thermodrucker TD40
;
; reass: Volker Pohlers 18.08.2008, letzte nderung: 26.08.2008
;------------------------------------------------------------------------------

		cpu	Z80
		page	0

;------------------------------------------------------------------------------
; OS
;------------------------------------------------------------------------------

; RAM
IOBYT		equ	0004h		; I/O-Byte zur Zuweisung log.Gert phy.	Gert
LISW		equ	0015h		; Schalter fr Hardcopy. 0 kein Copy, 1 Copy
KEYBU		equ	0025h		; Tastaturpuffer
EOR		equ	0036h		; Zeiger auf letzte fr	Anwender freie Adresse
SYSR		equ	02C0h		; reserviert fr Systemerweiterung
ATTYL		equ	0EFE1h		; Adresse TTY-Treiber fr LIST
TXLST		equ	0EFEFh		; Adresse einer	Zeichenkette des aktuellen LIST-Gertes
BWS		equ	0EC00h		; Bildwiederholspeicher

; CALL5
PRNST		equ 	09h		; Ausgabe einer	Zeichenkette zu	CONST
RCONB		equ 	0Ah		; Eingabe einer	Zeichenkette von CONST


;------------------------------------------------------------------------------
; Druckermodul
;------------------------------------------------------------------------------

SIODA		EQU	0B0H		; Datenadr SIO Kanal A
SIOKD		EQU	SIODA+2		; Kdoadr   - " -
CTC		EQU	0A8H		; Adr CTC Kanal 0

; Drucker mit Schnittstelle RS232C und DTR-Protokoll (Hardware-Protokoll)

; Baudrate:
;	BDRAT in Bd (50, 75, 100, 150, 200, 300, 600, 1200, 2400, 4800 oder 9600)
; 	Zeitkonstante fuer CTC: ZKCTC=9600/BDRAT
; Datenbit: DABI
;	7 Datenbit: DABI=20H
;	8 Datenbit: DABI=60H
; Paritaetsbit: PABI
;	ohne     Paritaet: PABI=2
;	gerade   Paritaet: PABI=3
;	ungerade Paritaet: PABI=1
; Stopbit: STPBI
;	1   Stopbit: STPBI=4
;	1.5 Stopbit: STPBI=8
;	2   Stopbit: STPBI=0CH

; 690006.8 TD40
ZKCTC		EQU	8		; 1200 Bd
DABI		EQU	20H		; 1 Datenbit
PABI		EQU	3		; gerade Paritaet
STPBI		EQU	4		; 1 Stopbit

; Zeichensatz: 
;	Steuerzeichen
; 		08H, 09H, 0AH, 0DH: wie auf Bildschirm
; 		0EH (CTRL/N, Bildschirmkopie) 10H (CTRL/P, Hardcopy-Modus)
; 		0CH, 17H Zeilenschaltung
; 	Druckzeichen
; 		Grafikdruck: wie beim Computer Z9001
; 		Alphanum. Druck: Grafikzeichen in "*"
; 			gewandelt, alphanum. Zeichen wie beim Z9001
; Zeichenraster: 
;	Grafikdruck: 6X8
; 	Alphanum. Druck: 6x10

;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

		org 0B800h

		jp	eos
		db "#       ",0

		jp	dini
name:		db "TD40    ",0
		db    0

;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

eos:		call	dini
		jp	0F089h		; Sprung ins OS

;Aufbau RAM-Speicher
;Lnge 170 Byte
;IX=(2C4)=(EOR)
;
;IX+0
;IX+1,2 akt. Pos. im Zeilenpuffer
;IX+3,4 Beginn Zeilenpuffer
;IX+5,6 Ende Zeilenpuffer
;IX+7,8 Merkzelle fr Zeilenpufferende (z.B. bei Wagenrcklauf)
;IX+9   Merkzelle fr Grafik (0Fh, 16h)
;IX+10  Merkzelle fr CRLF, FFh -> letzten Zeichen war CR, sonst 0
;IX+10..169 Zeilenpuffer

; max. Zeilenlnge 140
; Lnge Zeilenpuffer 160

;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

dini:		ld	hl, (TXLST)	; Zeichenkette f. LIST eintragen
		ld	de, name	; "TD40    "
		and	a		; Cy = 0
		sbc	hl, de
		ld	(TXLST), de
		push	de
		jr	z, ini1		; Druckertreiber schon init.
;
		ld	hl, (EOR)
		ld	de, 170
		and	a
		sbc	hl, de
		ld	(EOR), hl
		ld	(2C4h),	hl	; SYSR+4
		push	hl
		pop	ix
		ld	de, 10
		add	hl, de
		ld	(ix+4),	h	; Beginn Zeilenpuffer
		ld	(ix+3),	l
		ld	(ix+2),	h	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	l
		ld	de, 140
		add	hl, de
		ld	(ix+6),	h	; Ende Zeilenpuffer
		ld	(ix+5),	l

		ld	(ix+10),0	; CR-Merker rcksetzen
;Programmierung CTC und SIO
		ld	hl, PROGT
		ld	c, CTC		; Prog. CTC
		ld	b, 2
		otir
		ld	c, SIOKD	; Prog. SIO
		ld	b, 8
		otir
;
		call	clrzp		; Zeilenpuffer mit Leerzeichen init.
;
ini1:		ld	hl, ttyl	; TTY-Treiber f. LIST
		ld	(ATTYL), hl
		ld	hl, 4		; LIST := ATTYP
		res	7, (hl)
		res	6, (hl)
		xor	a
		ld	(LISW), a	; kein Hardcopy
		call	status		; Statusabfrage
		ld	c, PRNST	; Ausgabe einer	Zeichenkette zu	CONST
		pop	de
		call	nc, 5
		and	a		; Cy = 0, d.h. kein Fehler
		ret


;------------------------------------------------------------------------------
; Initialisierungstabelle CTC und SIO
;------------------------------------------------------------------------------

PROGT:
;CTC
		db 00010111b		; Zeitgeber/DI/Vorteiler 16
		db ZKCTC		; Zeitkonstante
;SIO
		db 4			; Steuerregister
		db 40H+PABI+STPBI	; keine	Paritt,1 Stoppbit,Vorteiler 16,Monosynch-Mode
		db 1			; Interruptvektor
		db 0			; Kein Interrupt
		db 3			; Steuerung Empfnger
		db 40h			; 7 bit, Empf. aus
		db 5			; Steuerung Sender
		db DABI+8		; 7 bit, Sender	ein

;------------------------------------------------------------------------------
; TTY-Treiber f. LIST (logischer Treiber)
;
; Treibereingangsparameter:
; - A Kommando (siehe unten)
; - C Zeichen (bei Kommando AUSGABE)
;
; Treiberausgangsparameter:
; - CY Fehlerstatus
;
; Kommando
; 00 Abfrage Status
; 	Return:
; 	A Status
; 	0 kein Zeichen bei Eingabegert, nicht bereit bei Ausgabegert
; 	sonst Zeichen liegt an bei Eingabegert,
; 	(im installierten CRT-Treiber wird der Zeichencode bergeben)
; 02 Ausgabe Zeichen
; 	Eingang:
; 	C Zeichen
; (FF Initialisieren/Rcksetzen des Gertes)
;
; Es wird nur 00 ausgewertet, alle anderen codes werden ignoriert -> Zeichenausgabe
; FF ist nicht implementiert
;------------------------------------------------------------------------------

ttyl:		and	a		; TTY-Treiber f. LIST
		jp	z, status	; wenn A=0 -> Statusabfrage
		push	ix
		ld	ix, (2C4h)	; SYSR+4
		ld	(ix+9),	0Fh
		ld	hl, ttylret
		push	hl		; RETURN-Adresse
		ld	a, c		; A=auszugebendes Zeichen
		ld	hl, ttyltab1	; Tabelle der Steuerzeichen
		ld	bc, 7
		cpir			; Steuerzeichen	suchen
		jr	z, ttyl1	; wenn A Steuerzeichen
		dec	bc		; bc :=	-1, wenn kein Steuerz., so Funktion char_xx
;
ttyl1:		ld	hl, 6
		and	a
		sbc	hl, bc
		add	hl, hl
		ld	bc, ttyltab2	; Tabelle der Steuerzeichen-Funktionen
		add	hl, bc
		ld	c, (hl)
		inc	hl
		ld	h, (hl)
		ld	l, c
		jp	(hl)		; Steuerzeichenfunktion	starten
;
ttylret:	pop	ix
		ret	nc
		ld	a, 0
		ld	(LISW), a	; kein Hardcopy
		ret

; Tabelle der Steuerzeichen
ttyltab1:	db 0Dh
		db 0Ah
		db 0Ch
		db 9
		db 17h
		db 0Eh
		db 8
; Tabelle der Steuerzeichen-Funktionen
ttyltab2:	dw char_0d		; Wagenrcklauf CR
		dw char_0a		; Cursor down
		dw char_0c		; CRLF (CLS?)
		dw char_09		; Cursor right
		dw char_0c		; CTRL/W Seitenvorschub	am Drucker
		dw char_0e		; CTRL/N Bildschirmkopie
		dw char_08		; Cursor left
		dw char_xx		; alle anderen Zeichen


;------------------------------------------------------------------------------
; Steuerzeichenfunktionen
;------------------------------------------------------------------------------

; Wagenrcklauf CR
char_0d:	ld	(ix+10),0FFh	; CR merken
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	(ix+8),	h	; merken
		ld	(ix+7),	l
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	(ix+2),	h	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	l
		and	a		; Cy := 0
		ret

; Cursor down LF
char_0a:	bit	0, (ix+10)	; war CR?
		jr	nz, char_0a_1	; wenn ja
;
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	(ix+8),	h	; merken
		ld	(ix+7),	l
char_0a_1:	ld	(ix+10),0	; CR-Merker rcksetzen
		call	prepzp
		jp	clrzp

; CLS -> CRLF
char_0c:	call	char_0d		; CR
		jp	char_0a		; LF

; Cursor right
char_09:	ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	d, (ix+6)	; Ende Zeilenpuffer
		ld	e, (ix+5)
		and	a		; Cy := 0
		sbc	hl, de		; Zeilenende erreicht?
		ccf
		call	z, char_0c	; ja -> CRLF
		ld	(ix+10),0	; CR-Merker rcksetzen
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		inc	hl
		ld	(ix+2),	h	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	l
		ret

; sonstige Zeichen
char_xx:	cp	20h
		jr	nc, char_xx_1	; Steuerzeichen < 20h bergehen
		and	a		; Cy := 0
		ret
;
char_xx_1:	push	af
		call	char_09		; Cursor right
		pop	de
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	(hl), d		; Zeichen im Zeilenpuffer eintragen
		ret


; CTRL/N Bildschirmkopie
char_0e:	call	char_0c		; CRLF
		ret	c
		ld	(ix+9),	16h
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	de, 40
		add	hl, de
		ld	(ix+6),	h	; Ende Zeilenpuffer
		ld	(ix+5),	l	; auf 40 Zeichen/Zeile verkleinern
;
		ld	de, BWS		; Bildwiederholspeicher
char_0e_1:	ld	a, (de)
		push	de
		call	char_xx		; Zeichen ausgeben (logisch)
					; Zeilenwechsel erfolgt automatisch
		pop	de
		ret	c
		inc	de
		ld	hl, BWS+40*24-1	; BWS-Ende
		and	a
		sbc	hl, de
		jr	nc, char_0e_1
;
		call	char_0c		; CRLF
		ret	c
;
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	de, 140
		add	hl, de
		ld	(ix+6),	h	; Ende Zeilenpuffer
		ld	(ix+5),	l	; auf 140 Zeichen/Zeile restaurieren
		ret

; Cursor left
char_08:	ld	(ix+10),0	; CR-Merker rcksetzen
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	d, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	e, (ix+1)
		and	a
		sbc	hl, de		; Am Anfang der Zeile?
		ret	z		; dann gibt es nicht zu tun
		dec	de		; sonst 1 Zeichen nach links
		ld	(ix+2),	d	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	e
		and	a
		ret


;------------------------------------------------------------------------------
; Zeilenpuffer-Routinen
;------------------------------------------------------------------------------

; Zeilenpuffer mit Leerzeichen init.
clrzp:		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	b, 160		; Lnge Zeilenpuffer
clrzp1:		inc	hl
		ld	(hl), ' '
		djnz	clrzp1
		ret

; Zeilenpuffer fr Ausdrucken vorbereiten
prepzp:		ld	a, (ix+9)
		ld	h, (ix+8)	; Merkzelle fr Zeilenpufferende
		ld	l, (ix+7)
		push	hl
		ld	b, (ix+4)	; Beginn Zeilenpuffer
		ld	c, (ix+3)
		and	a
		sbc	hl, bc
		ld	b, h		; BC := HL := Anzahl auszugebender Zeichen
		ld	c, l
		ex	(sp), hl	; HL := Zeilenpufferende
		push	hl
		jr	z, prepzp1	; wenn keine Zeichen auszugeben
; sonst Zeilenpuffer 2 Zeichen nach rechts verschieben
		ld	d, h
		ld	e, l
		inc	de
		inc	de
		lddr
;
prepzp1:	inc	hl
		ld	(hl), 0Dh	; als erstes Zeichen CR eintragen
		inc	hl
		ld	(hl), a
;
		pop	hl		; HL := Zeilenpufferende
prepzp2:	inc	hl
		inc	hl
		ex	(sp), hl
		ld	bc, 40		; Zeichen pro Zeile
		and	a
		sbc	hl, bc
		ld	b, h
		ld	c, l
		ex	(sp), hl
		jr	z, prepzp3
		jr	c, prepzp3
; wenn Zeilenlnge erreicht, so LFCR einfgen
		push	hl
		ld	d, h
		ld	e, l
		inc	de
		inc	de
		inc	de
		lddr
		inc	hl
		ld	(hl), 0Ah	; LF einfgen
		inc	hl
		ld	(hl), 0Dh	; CR einfgen
		inc	hl
		ld	(hl), a 		
		pop	hl
		inc	hl
		jr	prepzp2
;
prepzp3:	pop	de
		inc	hl
		ld	(hl), 0Ah	; LF einfgen
		ld	(ix+8),	h	; Merkzelle fr Zeilenpufferende
		ld	(ix+7),	l

; Ausdrucken einer Zeile
prnzp:		ld	b, (ix+4)	; Beginn Zeilenpuffer
		ld	c, (ix+3)
prnzp1:		ld	h, (ix+8)	; Merkzelle fr Zeilenpufferende
		ld	l, (ix+7)
		and	a
		sbc	hl, bc
		ret	z		; wenn kein Zeichen auszugeben ist
;
		ld	e, 80h
prnzp3:		push	bc
prnzp2:		inc	bc
		ld	a, (bc)
		push	bc
		call	prngr		; Grafikzeichenausgabe
		call	dout		; Ausgabe D
		pop	bc
		pop	hl
		ret	c
		push	hl
		ld	a, (bc)
		cp	0Ah
		jr	nz, prnzp2	; Zeichen ausgeben bis LF
;
		bit	4, (ix+9)
		jr	z, prnzp4
		srl	e
		jr	c, prnzp4
		pop	bc
		jr	prnzp3

prnzp4:		pop	hl
		jr	prnzp1



; Grafikzeichenausgabe
prngr:		ld	bc, 4
		ld	hl, grtab	; nicht	druckbare Steuerzeichen
		cpir
		ld	d, a
		ret	z
		bit	4, (ix+9)
		jr	z, nogr
		ld	l, a
		ld	h, 0
		ld	bc, 20h	; ' '
		and	a
		sbc	hl, bc
		add	hl, hl
		push	hl
		add	hl, hl
		pop	bc
		add	hl, bc
		ld	bc, chrset	; Grafikzeichensatz 6x8	(ab 20h)
		add	hl, bc
		ld	d, 0FFh
		ld	b, 6
prngr1:		ld	a, (hl)
		and	e
		jr	z, prngr2
		scf
prngr2:		rl	d
		inc	hl
		djnz	prngr1
		ret

nogr:		cp	7Fh
		ret	c
		ld	d, '*'
		ret

; nicht	druckbare Steuerzeichen
grtab:		db  0Dh			; ENTER
		db  0Fh			; CONTR-O 
		db  16h			; CONTR-V Umschaltung auf Invers
		db  0Ah			; cu down

;------------------------------------------------------------------------------
; phys. Treiber
;------------------------------------------------------------------------------

; Ausgabe D
dout:		call	stat		; lange	Statusabfrage
		ret	c
		ld	a, d
		out	(SIODA), a
		and	a
		ret

; Statusabfrage
status:		and	a
		ld	a, 10h
		out	(SIOKD), a
		in	a, (SIOKD)	; SIO RR0
		bit	5, a
		ret	nz
		scf
		ret

; lange	Statusabfrage
stat:		ld	b, 59		; Abfrage Status

; kurze	Statusabfrage
stat1:		ld	c, 10
stat2:		call	wait		; Zeitschleife
		call	status		; Statusabfrage
		ret	nc
		dec	c
		jr	nz, stat2
		djnz	stat1		; kurze	Statusabfrage
		scf
		ret

; Zeitschleife
wait:		push	bc
		ld	bc, 540h
wait1:		dec	c
		nop
		jr	nz, wait1
		djnz	wait1
		pop	bc
		ret


;------------------------------------------------------------------------------
; Zeichensatz 6x8 Pixel, 224 Zeichen ab 20h
;------------------------------------------------------------------------------

chrset:		equ	$
		include	zsatz_td40.inc

		end
